#!/bin/bash

# this script is intended to be run as "root" user.
if [ "`whoami`" != "root" ] ; then
	echo ">>> Execute init_routeserver.sh as a root user <<<" ; exit 1
fi

# Load configuration file and bail if it can't be read
. `dirname $0`/`basename $0 .sh`.conf || exit 1

# working directory
WORKING_DIR="$2"
# collection directory
COLLECTION_DIR="$HOMEPATH/$WORKING_DIR"
# location of the bgp configuration
BGP_CONFIG_DIR="$COLLECTION_DIR/bgp"
# daemon pid file
ZEBRA_PID=$BGP_CONFIG_DIR/zebra.pid
BGPD_PID=$BGP_CONFIG_DIR/bgpd.pid
# log file	
ROUTESERVER_LOG_FILE=$COLLECTION_DIR/log/Routeserver.log

if [ $# -gt 1 ]; then
	. "$COLLECTION_DIR/config/peering.cfg" || exit 1
else
	echo "Usage: $0 {start|stop|status} working_dir" ; exit 1
fi

if [ ! -f $ROUTESERVER_LOG_FILE ]; then
	su - $U -c "touch $ROUTESERVER_LOG_FILE" 
fi

# write log function
writeLog() {
	echo "[`date +%Y%m%d-%H:%M:%S`] $*" >> "$ROUTESERVER_LOG_FILE"	
	echo "[`date +%Y%m%d-%H:%M:%S`] $*"
}

doStart() {
	$QUAGGA -d -f "$BGP_CONFIG_DIR/zebra.conf" -i "$ZEBRA_PID" -P "$QUAGGA_VTY_PORT" -u $U -g $U
	$BGPD -d -f "$BGP_CONFIG_DIR/bgpd.conf" -i "$BGPD_PID" -p "$BGP_PORT" -P "$BGP_VTY_PORT" -n -u $U -g $U
}

doStop() {
	kill `cat "$BGPD_PID"` >& /dev/null && rm -rf $BGPD_PID || writeLog "  (Unable to kill bgpd)\n"
	kill `cat "$ZEBRA_PID"` >& /dev/null && rm -rf $ZEBRA_PID || writeLog "  (Unable to kill quagga/zebra)\n"
}

#Case
case "$1" in 
    start)
	
	if [ -f $BGPD_PID ]; then
  		oldpid_bgpd=$(cat $BGPD_PID)
  		otherpid_bgpd=$(ps h $oldpid_bgpd | grep -E "$BGP_CONFIG_DIR/bgpd.conf" | awk '{print $1;}')
	fi
	if [ -f $ZEBRA_PID ]; then
		oldpid_zebra=$(cat $ZEBRA_PID)
  		otherpid_zebra=$(ps h $oldpid_zebra | grep -E "$BGP_CONFIG_DIR/zebra.conf" | awk '{print $1;}')
	fi
  	if [ -n "$otherpid_bgpd" -a -n "$otherpid_zebra" ]; then
    		writeLog "$WORKING_DIR : BGPD and ZEBRA are already running with PID $oldpid_bgpd and $oldpid_zebra, exiting..."
    		exit 4
 	elif [ -z "$otherpid_bgpd" -a -z "$otherpid_zebra" ]; then
    		writeLog "$WORKING_DIR : BGPD and ZEBRA are not running, overwriting and continuing..."
		doStart
	else 
		writeLog "$WORKING_DIR : BGPD or ZEBRA is not running, restarting..."		
		doStop
		doStart
	fi	

 	;;  
	 
    stop)
	
	if [ -f $BGPD_PID ]; then
  		oldpid_bgpd=$(cat $BGPD_PID)
  		otherpid_bgpd=$(ps h $oldpid_bgpd | grep -E "$BGP_CONFIG_DIR/bgpd.conf" | awk '{print $1;}')
	fi
	if [ -f $ZEBRA_PID ]; then
		oldpid_zebra=$(cat $ZEBRA_PID)
  		otherpid_zebra=$(ps h $oldpid_zebra | grep -E "$BGP_CONFIG_DIR/zebra.conf" | awk '{print $1;}')
	fi
  	if [ -n "$otherpid_bgpd" -a -n "$otherpid_zebra" ]; then
    		writeLog "$WORKING_DIR : BGPD and ZEBRA are running with PID $oldpid_bgpd and $oldpid_zebra, stopping..."
		doStop
 	elif [ -z "$otherpid_bgpd" -a -z "$otherpid_zebra" ]; then
    		writeLog "$WORKING_DIR : BGPD and ZEBRA are not running, exiting..."
		exit 4
	else 
		writeLog "$WORKING_DIR : BGPD or ZEBRA is not running, stopping..."		
		doStop
	fi	
	
	;;
    status)
	

	if [ -f $BGPD_PID ]; then
  		oldpid_bgpd=$(cat $BGPD_PID)
  		otherpid_bgpd=$(ps h $oldpid_bgpd | grep -E "$BGP_CONFIG_DIR/bgpd.conf" | awk '{print $1;}')
	fi
	if [ -f $ZEBRA_PID ]; then
		oldpid_zebra=$(cat $ZEBRA_PID)
  		otherpid_zebra=$(ps h $oldpid_zebra | grep -E "$BGP_CONFIG_DIR/zebra.conf" | awk '{print $1;}')
	fi
  	if [ -n "$otherpid_bgpd" -a -n "$otherpid_zebra" ]; then
    		writeLog "$WORKING_DIR : BGPD and ZEBRA are running with PID $oldpid_bgpd and $oldpid_zebra."
		exit 4
 	elif [ -z "$otherpid_bgpd" -a -z "$otherpid_zebra" ]; then
    		writeLog "$WORKING_DIR : BGPD and ZEBRA are not running."
		exit 4
	else 
		writeLog "$WORKING_DIR : BGPD or ZEBRA is not running (BGPD pid $otherpid_bgpd, ZEBRA pid $otherpid_zebra)."		
		exit 4
	fi	

	;;
    restart)
	shift
	$0 stop $*
	$0 start $*
	;;
    *)
    	echo "Usage: $0 {start|stop|status} working_dir"
	exit 1
	;;
esac

