#!/bin/bash

# Handle return values for commands being run in a pipe chain properly
set -o pipefail

if [ $# -lt 1 ]; then
	echo "Usage: $0 working_dir" >&2
	exit 1
fi

RETRIEVER_INSTANCE="$1"

# Load configuration file and bail if it can't be read
. `dirname $0`/$RETRIEVER_INSTANCE/config/`basename $0 .sh`.conf || exit 1

# See if the config file specified alternate commands for accessing files.
[ -z "$LS" ] && LS=ls
[ -z "$CAT" ] && CAT=cat

# This is the name of the script
SCRIPT_NAME=`basename $0`

# write log function
writeLog() {
	echo "[`date +%Y%m%d-%H:%M:%S`] $*" >> "$RETRIEVER_LOG_FILE"
	echo "[`date +%Y%m%d-%H:%M:%S`] $*"
}

# Uncompress a file
uncompress_file() {
  filename=$1
  if [ "${filename: -4:4}" = ".bz2" ]; then
    $CAT $filename | bunzip2
  else
    if [ "${filename: -3:3}" = ".gz" ]; then
      $CAT $filename | gunzip
    else
      $CAT $filename
    fi
  fi
}

# proces record line function
process_rib_updates_line() {
  # Check whether we have to filter out "STATE" records from the MRT dump
  { if [ "$1" == "1" ]; then
      (grep -v "|STATE|") || true
  else
      cat
  fi; } | awk -F '|' -v OFS="|" '{
       split($0, record, "|")
       if ($3 != "W") {
           aspath=$7
           gsub(/( ?\{[0-9,]*\}$)+/,"",aspath)
           gsub(/ $/,"",aspath)
           for (i=2; i<=11; i++)
               printf "%s|", record[i]
           if (aspath=="")
               print $5"|"record[15]"|"record[16]"|"record[12]
           else
               print substr(aspath,match(aspath,/[0-9]*$/))"|"record[15]"|"record[16]"|"record[12]
       }
       else {
           for (i=2; i<=9; i++) {
                printf "%s", record[i]
                if (i<9)
                     printf "|"
		if (i==6)
                     printf "||||||"
           }
           printf "\n"
       }
   }'
}

process_state_line() {
  # The following "true" prevents returning a non-zero value when grep is
  # unable to find any STATE record
  { grep "|STATE|"; true; } | awk -F '|' -v OFS="|" '{
     split($0, record, "|")
     for (i=2; i<=7; i++) {
        if (i==3) continue
        printf "%s", record[i]
        if (i<7) printf "|"
     }
     printf "\n"
  }'
}

remove_pidfile() {
  rm -f $PIDFILE
  exit 1
}

writeLog ">>> Starting retrieving process"

SCRIPT_PID=$$
PIDFILE=`dirname $0`/$RETRIEVER_INSTANCE/retriever.pid

# Bash inserts in the process table an additional entry (with the same name as
# the script) for processing the following pipe.
if [ -f $PIDFILE ]; then
  oldpid=$(cat $PIDFILE)
  otherpid=$(ps h $oldpid | grep -E "$SCRIPT_NAME.*[ \t]+$RETRIEVER_INSTANCE" | awk '{print $1;}')
  if [ -n "$otherpid" ]; then
    writeLog "Another retriever is already running with PID $oldpid, exiting."
    exit 4
  else
    writeLog "Stale PID file with PID $oldpid, overwriting and continuing"
  fi
fi

trap remove_pidfile INT TERM QUIT EXIT

echo $SCRIPT_PID > $PIDFILE

#writeLog "Creating peering state table"

# Create table to store peering states
#mysql -u $DB_USER -h $DB_HOST $DB_NAME --password=$DB_PASSWORD -e "
#   CREATE TABLE IF NOT EXISTS peering_state (             \
#      time INT UNSIGNED default NULL,                     \
#      ippeer VARCHAR(40) NOT NULL,                        \
#      aspeer INT UNSIGNED default NULL,                   \
#      state_from TINYINT UNSIGNED default NULL,           \
#      state_to TINYINT UNSIGNED default NULL,
#      id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,\
#                                                          \
#      INDEX ippeer_index USING HASH (ippeer ASC)) DATA DIRECTORY = '$DATA_DIR' INDEX DIRECTORY = '$INDEX_DIR';
#   "
#

writeLog "Building the file list to process"

filelist=""
for path in $VM_DUMP_RIB_PATHS $VM_DUMP_UPDATES_PATHS; do
  filelist="$filelist $($LS $path/updates.????????.????{,.gz,.bz2}\
                        $path/rib.????????.0000{,.gz,.bz2} 2> /dev/null)"
done

# Return up to the last two path elements of the given path.
# i.e., /path/to/file -> to/file ; /file -> /file ; file -> file
function basepath {
    if [ "$PRESERVE_DIR" = "1" ]; then
        echo $1 | sed -re 's/.*\/([^/]+\/)([^/]+)/\1\2/'
    else
        basename $i
    fi
}

for i in $filelist; do 
    date=`basename $i | awk '{split($0,a,"."); print a[2]}'`
    kind=`basename $i | awk '{split($0,a,"."); print a[1]}'`

    LSOF=`whereis lsof | awk '{print $2}'`
    if [ -z "$LSOF" ]; then
         writeLog "WARNING: could not automatically determine the full path to lsof. VERY BAD THINGS MAY HAPPEN! Updates could be missed!!!"
         LSOF=/usr/sbin/lsof
    fi
    if sudo $LSOF $i >&/dev/null; then
          writeLog "Not processing (in use): $i"
	       continue
    fi
    if [ -f "$ARCHIVES_PATH/$date/`basepath $i`.bz2" ]; then
      writeLog "File $i already processed, skipping"
      continue
    fi

    writeLog "Processing $i"

# Max: modificato per supporto a VPLS e VPN MPLS
#    if [ "$kind" = "rib" ]; then
#      tablename="rib$date"
#      filter_state=0
#    elif [ "$kind" = "updates" ]; then
#      tablename="upd$date"
#      filter_state=1
#      else
#          writeLog "File $i have an invalid name, skipping"
#          continue
#      fi
    tablename="vpn_updates"
  
    writeLog "Converting from MRT to ASCII file $i which contains $kind related to date $date"

    temp_file_path=$HUMAN_READABLE_FILE_PATH/$(dirname $(basepath $i))
    if ! [ -d $temp_file_path ]; then
      writeLog "Creating directory $temp_file_path"
      mkdir -p $temp_file_path
    fi

   # uncompress_file "$i" | $BGPDUMP_PATH/bgpdump -m - >> "$RETRIEVER_LOG_FILE" 2>&1 > $HUMAN_READABLE_FILE_PATH/`basepath $i` || \
   #{
#		writeLog "ERROR: Bgpdump has failed on file $i (return status is $?)";
#	}
#		
#		writeLog "Create table if not exists $tablename"
	
cat "$i" | $BGPDUMP_PATH/bgpdump -m -> $HUMAN_READABLE_FILE_PATH/`basepath $i`

     # I campi originatorid e clusterlist vengono comunque creati anche nella RIB. 
     # Assumono ovviamente nel caso della RIB sempre il valore NULL, non essendo presenti nell'MRT
#     mysql -u $DB_USER -h $DB_HOST $DB_NAME --password=$DB_PASSWORD -e "
#         CREATE  TABLE IF NOT EXISTS $tablename (               \
#            time INT UNSIGNED default NULL,                     \
#            type ENUM('A','W','B') default NULL,                \
#            ippeer VARCHAR(40) NOT NULL,                        \
#            aspeer INT UNSIGNED default NULL,                   \
#	    prefix VARCHAR(44) default NULL,                    \
#	    aspath VARCHAR(255) default NULL,                   \
#	    origin ENUM('EGP','IGP','INCOMPLETE') default NULL, \
#	    nexthop VARCHAR(40) default NULL,                   \
 #           localpref INT UNSIGNED default NULL,                \
  #          med INT UNSIGNED default NULL,                      \
#	    asowner INT UNSIGNED default NULL,			\
#	    originatorid VARCHAR(40) default NULL,		\
#	    clusterlist VARCHAR(255) default NULL,		\
#	    community VARCHAR(255) default NULL,		\
                                                                \
 #           INDEX a_index USING HASH (asowner ASC),             \
  #          INDEX b_index USING HASH (prefix ASC),              \
   #         INDEX c_index USING HASH (ippeer ASC),              \
    #        INDEX d_index USING HASH (nexthop ASC),             \
     #       INDEX e_index USING BTREE (time ASC)) DATA DIRECTORY = '$DATA_DIR' INDEX DIRECTORY = '$INDEX_DIR';
    # "	
    
#estensione vpls
	mysql -u $DB_USER -h $DB_HOST $DB_NAME --password=$DB_PASSWORD -e "
         CREATE  TABLE IF NOT EXISTS $tablename (
            id INT NOT NULL AUTO_INCREMENT,
            type_packet VARCHAR(40) NOT NULL,
            type ENUM('A','W','B') default NULL,
            time INT UNSIGNED default NULL,
            ippeer VARCHAR(40) NOT NULL,
            aspeer INT UNSIGNED default NULL,
            origin ENUM('EGP','IGP','INCOMPLETE') default NULL,
            prefix_nlri VARCHAR(43) default NULL,
            nexthop VARCHAR(40) default NULL,
            nexthop_rd_type INT UNSIGNED default NULL,
            nexthop_rd VARCHAR(255) default NULL,
            vrf_target VARCHAR(40) default NULL,
            rd_type INT UNSIGNED default NULL,
            route_distinguisher VARCHAR(255) default NULL,
            rd_number VARCHAR(20) default NULL,
            ce_id VARCHAR(40) default NULL,
            labels_block_offset INT UNSIGNED default NULL,
            label_block_size INT UNSIGNED default NULL,
            label_base INT UNSIGNED default NULL,

            PRIMARY KEY (id),

            INDEX a_index USING HASH (ce_id ASC),
            INDEX b_index USING HASH (route_distinguisher ASC),
            INDEX c_index USING HASH (prefix_nlri ASC),
            INDEX d_index USING HASH (nexthop ASC),
            INDEX e_index USING BTREE (time ASC),	
            INDEX f_index USING BTREE (vrf_target ASC));"	
	#cambiato il path dell'ultimo index		
      writeLog "Inserting $kind data from $i into $tablename table"
      
      cat $HUMAN_READABLE_FILE_PATH/`basepath $i` | $MYSQL_LOAD_DATA -h $DB_HOST -u $DB_USER --password=$DB_PASSWORD $DB_NAME -e "LOAD DATA CONCURRENT LOCAL INFILE '/dev/stdin' INTO TABLE $tablename FIELDS TERMINATED BY '|' (type_packet,type,time,ippeer,aspeer,origin,prefix_nlri,nexthop,nexthop_rd_type,nexthop_rd,vrf_target,rd_type,route_distinguisher,rd_number,ce_id,labels_block_offset,label_block_size,label_base)"
      if [ $? != 0 ]; then
         writeLog "Error while loading $kind data from $i into table $tablename"
      fi

#      if [ $kind == "updates" ]; then
#         writeLog "Inserting peering_state"
#	 cat $HUMAN_READABLE_FILE_PATH/`basepath $i` | process_state_line | $MYSQL_LOAD_DATA -h $DB_HOST -u $DB_USER --password=$DB_PASSWORD $DB_NAME -e "LOAD DATA CONCURRENT LOCAL INFILE '/dev/stdin' INTO TABLE peering_state FIELDS TERMINATED BY '|'"
#         if [ $? != 0 ]; then
#            writeLog "Error while loading peering state data from $i into table peering_state"
#         fi
#      fi
 
     rm -f "$HUMAN_READABLE_FILE_PATH"/`basepath $i` || writeLog "Error while removing files from $HUMAN_READABLE_FILE_PATH"

      writeLog "Compressing and moving $i to $ARCHIVES_PATH/$date"
      processed_file_path=$ARCHIVES_PATH/$date/$(dirname $(basepath $i))
      if ! [ -d $processed_file_path ]; then
        writeLog "Creating directory $processed_file_path"
        mkdir -p $processed_file_path
      fi

      if [ -z "$NO_ARCHIVE" ]; then
        { uncompress_file "$i" | bzip2 > $ARCHIVES_PATH/$date/`basepath $i`.bz2; rm -f $i; } || writeLog "Error while compressing and/or removing $i"
      else
        touch $ARCHIVES_PATH/$date/`basepath $i`.bz2
        rm -f $i
      fi

done

#writeLog "Removing useless RIBs from $VM_DUMP_RIB_PATHS"
#rm -f $VM_DUMP_RIB_PATHS/rib.????????.1200 || writeLog "Error while removing useless RIBs"

writeLog ">>> End of retrieving process"

